@@ -211,7 +211,7 @@ func TestGenerateObjectName(t *testing.T) {
 			cache:              cache,
 			build:              cacheBuild,
 			key:                "key",
-			expectedObjectName: "runner/longtoke/project/10/key",
+			expectedObjectName: "runner/longtoken/project/10/key",
 		},
 		"empty key": {
 			cache:              cache,
@@ -224,21 +224,21 @@ func TestGenerateObjectName(t *testing.T) {
 			build:              cacheBuild,
 			key:                "key",
 			path:               "whatever",
-			expectedObjectName: "whatever/runner/longtoke/project/10/key",
+			expectedObjectName: "whatever/runner/longtoken/project/10/key",
 		},
 		"multiple segment path is set": {
 			cache:              cache,
 			build:              cacheBuild,
 			key:                "key",
 			path:               "some/other/path/goes/here",
-			expectedObjectName: "some/other/path/goes/here/runner/longtoke/project/10/key",
+			expectedObjectName: "some/other/path/goes/here/runner/longtoken/project/10/key",
 		},
 		"path is empty": {
 			cache:              cache,
 			build:              cacheBuild,
 			key:                "key",
 			path:               "",
-			expectedObjectName: "runner/longtoke/project/10/key",
+			expectedObjectName: "runner/longtoken/project/10/key",
 		},
 		"shared flag is set to true": {
 			cache:              cache,
@@ -252,13 +252,13 @@ func TestGenerateObjectName(t *testing.T) {
 			build:              cacheBuild,
 			key:                "key",
 			shared:             false,
-			expectedObjectName: "runner/longtoke/project/10/key",
+			expectedObjectName: "runner/longtoken/project/10/key",
 		},
 		"path traversal but within base path": {
 			cache:              cache,
 			build:              cacheBuild,
 			key:                "../10/key",
-			expectedObjectName: "runner/longtoke/project/10/key",
+			expectedObjectName: "runner/longtoken/project/10/key",
 		},
 		"path traversal resolves to empty key": {
 			cache:         cache,
