@@ -19,6 +19,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/network"
 	"gitlab.com/gitlab-org/gitlab-runner/shells"
+	"gitlab.com/gitlab-org/labkit/fips"
 
 	// Force to load shell executor, executes init() on them
 	_ "gitlab.com/gitlab-org/gitlab-runner/executors/custom"
@@ -178,7 +179,7 @@ func (s *RegisterCommand) askExecutor() {
 }
 
 func (s *RegisterCommand) askDocker() {
-	s.askBasicDocker("ruby:2.7")
+	s.askBasicDocker("ruby:3.3")
 
 	for _, volume := range s.Docker.Volumes {
 		parts := strings.Split(volume, ":")
@@ -408,6 +409,20 @@ func (s *RegisterCommand) askExecutorOptions() {
 	}
 }
 
+// Set helper_image_flavor to ubi-fips if fips is enabled. See
+// https://gitlab.com/gitlab-org/gitlab-runner/-/issues/38273
+func setFipsHelperImageFlavor(cfg *common.RunnerConfig, fipsEnabled func() bool) {
+	if cfg == nil || !fipsEnabled() {
+		return
+	}
+	if cfg.Docker != nil && cfg.Docker.HelperImageFlavor == "" {
+		cfg.Docker.HelperImageFlavor = "ubi-fips"
+	}
+	if cfg.Kubernetes != nil && cfg.Kubernetes.HelperImageFlavor == "" {
+		cfg.Kubernetes.HelperImageFlavor = "ubi-fips"
+	}
+}
+
 func (s *RegisterCommand) Execute(context *cli.Context) {
 	userModeWarning(true)
 
@@ -432,6 +447,8 @@ func (s *RegisterCommand) Execute(context *cli.Context) {
 		s.askExecutor()
 		s.askExecutorOptions()
 
+		setFipsHelperImageFlavor(&s.RunnerConfig, fips.Enabled)
+
 		config.Runners = append(config.Runners, &s.RunnerConfig)
 		return nil
 	})); err != nil {
