@@ -208,8 +208,12 @@ func (p *provider) init(config *common.RunnerConfig) (taskscaler.Taskscaler, boo
 		taskscaler.WithMetricsCollector(tsMC),
 		taskscaler.WithFleetingMetricsCollector(flMC),
 		taskscaler.WithInstanceUpFunc(instanceReadyUp(shutdownCtx, config)),
-		taskscaler.WithUpdateInterval(time.Minute),
-		taskscaler.WithUpdateIntervalWhenExpecting(time.Second),
+		taskscaler.WithUpdateInterval(fleeting.DefaultUpdateInterval),
+		taskscaler.WithUpdateIntervalWhenExpecting(fleeting.DefaultUpdateIntervalWhenExpecting),
+		taskscaler.WithDeletionRetryInterval(fleeting.DefaultDeletionRetryInterval),
+		taskscaler.WithShutdownDeletionInterval(fleeting.DefaultShutdownDeletionInterval),
+		taskscaler.WithShutdownDeletionRetries(fleeting.DefaultShutdownDeletionRetries),
+		taskscaler.WithFailureThreshold(3),
 		taskscaler.WithLogger(logger.Named("taskscaler")),
 		taskscaler.WithScaleThrottle(config.Autoscaler.ScaleThrottle.Limit, config.Autoscaler.ScaleThrottle.Burst),
 	}
@@ -230,6 +234,22 @@ func (p *provider) init(config *common.RunnerConfig) (taskscaler.Taskscaler, boo
 		options = append(options, taskscaler.WithUpdateIntervalWhenExpecting(config.Autoscaler.UpdateIntervalWhenExpecting))
 	}
 
+	if config.Autoscaler.DeletionRetryInterval > 0 {
+		options = append(options, taskscaler.WithDeletionRetryInterval(config.Autoscaler.DeletionRetryInterval))
+	}
+
+	if config.Autoscaler.ShutdownDeletionInterval > 0 {
+		options = append(options, taskscaler.WithShutdownDeletionInterval(config.Autoscaler.ShutdownDeletionInterval))
+	}
+
+	if config.Autoscaler.ShutdownDeletionRetries > 0 {
+		options = append(options, taskscaler.WithShutdownDeletionRetries(config.Autoscaler.ShutdownDeletionRetries))
+	}
+
+	if config.Autoscaler.FailureThreshold > 0 {
+		options = append(options, taskscaler.WithFailureThreshold(config.Autoscaler.FailureThreshold))
+	}
+
 	if config.Autoscaler.DeleteInstancesOnShutdown {
 		options = append(options, taskscaler.WithDeleteInstancesOnShutdown())
 	}
