@@ -5,7 +5,7 @@ package vault
 import (
 	"testing"
 
-	"github.com/hashicorp/vault/api"
+	"github.com/openbao/openbao/api/v2"
 	"github.com/stretchr/testify/assert"
 )
 
