@@ -19,12 +19,10 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
-	"strconv"
 	"strings"
 	"testing"
 	"time"
 
-	"github.com/jpillora/backoff"
 	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/mock"
@@ -123,7 +121,7 @@ func writeTLSKeyPair(s *httptest.Server, certFile, keyFile string) error {
 func TestNewClient(t *testing.T) {
 	c, err := newClient(&RunnerCredentials{
 		URL: "http://test.example.com/ci///",
-	})
+	}, NewAPIRequestsCollector())
 	assert.NoError(t, err)
 	assert.NotNil(t, c)
 	assert.Equal(t, "http://test.example.com/api/v4/", c.url.String())
@@ -132,7 +130,7 @@ func TestNewClient(t *testing.T) {
 func TestInvalidUrl(t *testing.T) {
 	_, err := newClient(&RunnerCredentials{
 		URL: "address.com/ci///",
-	})
+	}, NewAPIRequestsCollector())
 	assert.Error(t, err)
 }
 
@@ -166,7 +164,7 @@ func TestServerCertificateChange(t *testing.T) {
 	// create runner client
 	c, err := newClient(&RunnerCredentials{
 		URL: "https://" + ln.Addr().String(),
-	})
+	}, NewAPIRequestsCollector())
 	require.NoError(t, err)
 	require.NotNil(t, c)
 
@@ -209,7 +207,7 @@ func TestClientDo(t *testing.T) {
 
 	c, err := newClient(&RunnerCredentials{
 		URL: s.URL,
-	})
+	}, NewAPIRequestsCollector())
 	assert.NoError(t, err)
 	assert.NotNil(t, c)
 
@@ -322,7 +320,7 @@ func TestClientNilBody(t *testing.T) {
 
 	c, err := newClient(&RunnerCredentials{
 		URL: s.URL,
-	})
+	}, NewAPIRequestsCollector())
 	assert.NoError(t, err)
 	assert.NotNil(t, c)
 
@@ -351,7 +349,7 @@ func TestClientDo_Context(t *testing.T) {
 
 	c, err := newClient(&RunnerCredentials{
 		URL: "http://gitlab.example.com",
-	})
+	}, NewAPIRequestsCollector())
 	require.NoError(t, err)
 	require.NotNil(t, c)
 
@@ -374,7 +372,7 @@ func TestClientInvalidSSL(t *testing.T) {
 
 	c, _ := newClient(&RunnerCredentials{
 		URL: s.URL,
-	})
+	}, NewAPIRequestsCollector())
 	statusCode, statusText, _ := c.doJSON(
 		context.Background(),
 		"test/ok",
@@ -405,7 +403,7 @@ func TestClientTLSCAFile(t *testing.T) {
 	c, _ := newClient(&RunnerCredentials{
 		URL:       s.URL,
 		TLSCAFile: file.Name(),
-	})
+	}, NewAPIRequestsCollector())
 	statusCode, statusText, resp := c.doJSON(
 		context.Background(),
 		"test/ok",
@@ -439,7 +437,7 @@ func TestClientCertificateInPredefinedDirectory(t *testing.T) {
 
 	c, _ := newClient(&RunnerCredentials{
 		URL: s.URL,
-	})
+	}, NewAPIRequestsCollector())
 	statusCode, statusText, resp := c.doJSON(
 		context.Background(),
 		"test/ok",
@@ -474,7 +472,7 @@ func TestClientInvalidTLSAuth(t *testing.T) {
 	c, _ := newClient(&RunnerCredentials{
 		URL:       s.URL,
 		TLSCAFile: ca.Name(),
-	})
+	}, NewAPIRequestsCollector())
 	statusCode, statusText, _ := c.doJSON(
 		context.Background(),
 		"test/ok",
@@ -521,7 +519,7 @@ func TestClientTLSAuth(t *testing.T) {
 		TLSCAFile:   ca.Name(),
 		TLSCertFile: cert.Name(),
 		TLSKeyFile:  key.Name(),
-	})
+	}, NewAPIRequestsCollector())
 
 	statusCode, statusText, resp := c.doJSON(
 		context.Background(),
@@ -568,7 +566,7 @@ func TestClientTLSAuthCertificatesInPredefinedDirectory(t *testing.T) {
 
 	c, _ := newClient(&RunnerCredentials{
 		URL: s.URL,
-	})
+	}, NewAPIRequestsCollector())
 	statusCode, statusText, resp := c.doJSON(
 		context.Background(),
 		"test/ok",
@@ -629,7 +627,7 @@ func TestClientHandleCharsetInContentType(t *testing.T) {
 
 	c, _ := newClient(&RunnerCredentials{
 		URL: s.URL,
-	})
+	}, NewAPIRequestsCollector())
 
 	res := struct {
 		Key string `json:"key"`
@@ -680,68 +678,10 @@ func TestClientHandleCharsetInContentType(t *testing.T) {
 	assert.Equal(t, -1, statusCode, statusText)
 }
 
-type backoffTestCase struct {
-	responseStatus int
-	mustBackoff    bool
-}
-
-func tooManyRequestsHandler(w http.ResponseWriter, r *http.Request) {
-	status, err := strconv.Atoi(r.Header.Get("responseStatus"))
-	if err != nil {
-		w.WriteHeader(599)
-	} else {
-		w.WriteHeader(status)
-	}
-}
-
-func TestRequestsBackOff(t *testing.T) {
-	s := httptest.NewServer(http.HandlerFunc(tooManyRequestsHandler))
-	defer s.Close()
-
-	c, _ := newClient(&RunnerCredentials{
-		URL: s.URL,
-	})
-
-	testCases := []backoffTestCase{
-		{http.StatusCreated, false},
-		{http.StatusInternalServerError, true},
-		{http.StatusBadGateway, true},
-		{http.StatusServiceUnavailable, true},
-		{http.StatusOK, false},
-		{http.StatusConflict, true},
-		{http.StatusTooManyRequests, false}, // Backoff handled by caller based on header value
-		{http.StatusCreated, false},
-		{599, true},
-		{499, true},
-	}
-
-	backoff := c.ensureBackoff(http.MethodPost, "")
-	for id, testCase := range testCases {
-		t.Run(fmt.Sprintf("%d-%d", id, testCase.responseStatus), func(t *testing.T) {
-			backoff.Reset()
-			assert.Zero(t, backoff.Attempt())
-
-			headers := make(http.Header)
-			headers.Add("responseStatus", strconv.Itoa(testCase.responseStatus))
-
-			res, err := c.do(context.Background(), "/", http.MethodPost, nil, "application/json", headers)
-
-			assert.NoError(t, err)
-			assert.Equal(t, testCase.responseStatus, res.StatusCode)
-
-			var expected float64
-			if testCase.mustBackoff {
-				expected = 1.0
-			}
-			assert.Equal(t, expected, backoff.Attempt())
-		})
-	}
-}
-
 func TestRequesterCalled(t *testing.T) {
 	c, _ := newClient(&RunnerCredentials{
 		URL: "http://localhost:1000/",
-	})
+	}, NewAPIRequestsCollector())
 
 	rl := newMockRequester(t)
 
@@ -844,8 +784,7 @@ func Test307and308Redirections(t *testing.T) {
 				require.NoError(t, err)
 
 				c := &client{
-					url:             u,
-					requestBackOffs: make(map[string]*backoff.Backoff),
+					url: u,
 				}
 				c.requester = &c.Client
 
@@ -885,8 +824,7 @@ func TestEnsureUserAgentAlwaysSent(t *testing.T) {
 			require.NoError(t, err)
 
 			c := &client{
-				url:             url,
-				requestBackOffs: make(map[string]*backoff.Backoff),
+				url: url,
 			}
 			c.requester = &c.Client
 
