@@ -16,6 +16,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/google/uuid"
 	"github.com/sirupsen/logrus"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
@@ -27,7 +28,6 @@ const (
 	// createdRunnerTokenPrefix is the token prefix used for GitLab UI-created runner authentication tokens
 	createdRunnerTokenPrefix = "glrt-"
 	clientError              = -100
-	retryAfterHeader         = "Retry-After"
 	responseBodyPeekMax      = 512
 
 	correlationIDHeader = "X-Request-Id"
@@ -45,7 +45,7 @@ type GitLabClient struct {
 	connectionMaxAge     time.Duration
 }
 
-func (n *GitLabClient) getClient(credentials requestCredentials) (c *client, err error) {
+func (n *GitLabClient) getClient(credentials requestCredentials) (*client, error) {
 	n.lock.Lock()
 	defer n.lock.Unlock()
 
@@ -59,18 +59,19 @@ func (n *GitLabClient) getClient(credentials requestCredentials) (c *client, err
 		credentials.GetTLSCAFile(),
 		credentials.GetTLSCertFile(),
 	)
-	c = n.clients[key]
-	if c == nil {
-		c, err = newClient(
-			credentials,
-			WithMaxAge(n.connectionMaxAge))
-		if err != nil {
-			return
-		}
-		n.clients[key] = c
+	c, ok := n.clients[key]
+	if ok {
+		return c, nil
 	}
 
-	return
+	c, err := newClient(credentials, n.apiRequestsCollector, WithMaxAge(n.connectionMaxAge))
+	if err != nil {
+		return nil, fmt.Errorf("new client: %w", err)
+	}
+
+	n.clients[key] = c
+
+	return c, nil
 }
 
 func (n *GitLabClient) getLastUpdate(credentials requestCredentials) (lu string) {
@@ -146,7 +147,7 @@ func (n *GitLabClient) doMeasuredRaw(
 	var response *http.Response
 	var err error
 
-	fn := func() int {
+	fn := func() (int, string) {
 		// Response body is handled after doMeasuredJSON() decorator call
 		// Linting violation here is a false-positive.
 		// nolint:bodyclose
@@ -160,10 +161,10 @@ func (n *GitLabClient) doMeasuredRaw(
 			params.headers,
 		)
 		if err != nil {
-			return clientError
+			return clientError, ""
 		}
 
-		return response.StatusCode
+		return response.StatusCode, params.method
 	}
 
 	n.apiRequestsCollector.Observe(
@@ -174,7 +175,11 @@ func (n *GitLabClient) doMeasuredRaw(
 		fn,
 	)
 
-	return response, err
+	if err != nil {
+		return nil, fmt.Errorf("measured raw request: %w", err)
+	}
+
+	return response, nil
 }
 
 func (n *GitLabClient) doRaw(
@@ -187,10 +192,15 @@ func (n *GitLabClient) doRaw(
 ) (res *http.Response, err error) {
 	c, err := n.getClient(credentials)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("get client: %w", err)
 	}
 
-	return c.do(ctx, uri, method, bodyProvider, requestType, headers)
+	response, err := c.do(ctx, uri, method, bodyProvider, requestType, headers)
+	if err != nil {
+		return nil, fmt.Errorf("execute raw request: %w", err)
+	}
+
+	return response, nil
 }
 
 type doJSONParams struct {
@@ -217,7 +227,7 @@ func (n *GitLabClient) doMeasuredJSON(
 	var statusText string
 	var httpResponse *http.Response
 
-	fn := func() int {
+	fn := func() (int, string) {
 		// Response body is handled after doMeasuredJSON() decorator call
 		// Linting violation here is a false-positive.
 		// nolint:bodyclose
@@ -232,7 +242,7 @@ func (n *GitLabClient) doMeasuredJSON(
 			params.response,
 		)
 
-		return result
+		return result, params.method
 	}
 
 	n.apiRequestsCollector.Observe(
@@ -273,6 +283,18 @@ func RunnerTokenHeader(runnerToken string) http.Header {
 	return headers
 }
 
+// addCorrelationID to passed in http.Header. If a nil value
+// is passed, a new instance of http.Header is created and
+// correlation ID is added to it.
+func addCorrelationID(headers http.Header) (http.Header, string) {
+	if headers == nil {
+		headers = http.Header{}
+	}
+	correlationID := strings.ReplaceAll(uuid.NewString(), "-", "")
+	headers.Set(correlationIDHeader, correlationID)
+	return headers, correlationID
+}
+
 func (n *GitLabClient) doJSON(
 	ctx context.Context,
 	credentials requestCredentials,
@@ -284,7 +306,7 @@ func (n *GitLabClient) doJSON(
 ) (int, string, *http.Response) {
 	c, err := n.getClient(credentials)
 	if err != nil {
-		return clientError, err.Error(), nil
+		return clientError, fmt.Errorf("get client: %w", err).Error(), nil
 	}
 
 	return c.doJSON(ctx, uri, method, statusCode, headers, request, response)
@@ -318,6 +340,8 @@ func (n *GitLabClient) RegisterRunner(
 		Info:                     n.getRunnerVersion(common.RunnerConfig{}),
 	}
 
+	headers, correlationID := addCorrelationID(RunnerTokenHeader(runner.Token))
+
 	var response common.RegisterRunnerResponse
 	result, statusText, resp := n.doJSON(
 		context.Background(),
@@ -325,13 +349,13 @@ func (n *GitLabClient) RegisterRunner(
 		http.MethodPost,
 		"runners",
 		http.StatusCreated,
-		RunnerTokenHeader(runner.Token),
+		headers,
 		&request,
 		&response,
 	)
-	defer func() { n.handleResponse(context.TODO(), resp, false) }()
+	defer closeResponseBody(resp, false)
 
-	logger := runner.Log().WithField("correlation_id", getCorrelationId(resp))
+	logger := runner.Log().WithField("correlation_id", getCorrelationID(resp, correlationID))
 
 	switch result {
 	case http.StatusCreated:
@@ -355,33 +379,39 @@ func (n *GitLabClient) VerifyRunner(runner common.RunnerCredentials, systemID st
 		SystemID: systemID,
 	}
 
+	headers, correlationID := addCorrelationID(RunnerTokenHeader(runner.Token))
+
 	var response common.VerifyRunnerResponse
+	//nolint:bodyclose
+	// body is closed with closeResponseBody function call
 	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodPost,
 		"runners/verify",
 		http.StatusOK,
-		RunnerTokenHeader(runner.Token),
+		headers,
 		&request,
 		&response,
 	)
 	if result == -1 {
 		// if server is not able to return JSON, let's try plain text (the legacy response format)
+		//nolint:bodyclose
+		// body is closed with closeResponseBody function call
 		result, statusText, resp = n.doJSON(
 			context.Background(),
 			&runner,
 			http.MethodPost,
 			"runners/verify",
 			http.StatusOK,
-			RunnerTokenHeader(runner.Token),
+			headers,
 			&request,
 			nil,
 		)
 	}
-	defer func() { n.handleResponse(context.TODO(), resp, false) }()
+	defer closeResponseBody(resp, false)
 
-	logger := runner.Log().WithField("correlation_id", getCorrelationId(resp))
+	logger := runner.Log().WithField("correlation_id", getCorrelationID(resp, correlationID))
 
 	switch result {
 	case http.StatusOK:
@@ -413,19 +443,21 @@ func (n *GitLabClient) UnregisterRunner(runner common.RunnerCredentials) bool {
 		Token: runner.Token,
 	}
 
+	headers, correlationID := addCorrelationID(RunnerTokenHeader(runner.Token))
+
 	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodDelete,
 		"runners",
 		http.StatusNoContent,
-		RunnerTokenHeader(runner.Token),
+		headers,
 		&request,
 		nil,
 	)
-	defer func() { n.handleResponse(context.TODO(), resp, false) }()
+	defer closeResponseBody(resp, false)
 
-	logger := runner.Log().WithField("correlation_id", getCorrelationId(resp))
+	logger := runner.Log().WithField("correlation_id", getCorrelationID(resp, correlationID))
 
 	const baseLogText = "Unregistering runner from GitLab"
 	switch result {
@@ -450,19 +482,21 @@ func (n *GitLabClient) UnregisterRunnerManager(runner common.RunnerCredentials,
 		SystemID: systemID,
 	}
 
+	headers, correlationID := addCorrelationID(RunnerTokenHeader(runner.Token))
+
 	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodDelete,
 		"runners/managers",
 		http.StatusNoContent,
-		RunnerTokenHeader(runner.Token),
+		headers,
 		&request,
 		nil,
 	)
-	defer func() { n.handleResponse(context.TODO(), resp, false) }()
+	defer closeResponseBody(resp, false)
 
-	logger := runner.Log().WithField("correlation_id", getCorrelationId(resp))
+	logger := runner.Log().WithField("correlation_id", getCorrelationID(resp, correlationID))
 
 	const baseLogText = "Unregistering runner manager from GitLab"
 	switch result {
@@ -506,8 +540,9 @@ func (n *GitLabClient) resetToken(
 		}
 	}
 
-	var response common.ResetTokenResponse
+	headers, correlationID := addCorrelationID(PrivateTokenHeader(pat))
 
+	var response common.ResetTokenResponse
 	result, statusText, resp := n.doMeasuredJSON(
 		context.Background(),
 		runner.Log(),
@@ -519,15 +554,15 @@ func (n *GitLabClient) resetToken(
 			method:      http.MethodPost,
 			uri:         uri,
 			statusCode:  http.StatusCreated,
-			headers:     PrivateTokenHeader(pat),
+			headers:     headers,
 			request:     request,
 			response:    &response,
 		},
 	)
 
-	defer func() { n.handleResponse(context.TODO(), resp, false) }()
+	defer closeResponseBody(resp, false)
 
-	logger := runner.Log().WithField("correlation_id", getCorrelationId(resp))
+	logger := runner.Log().WithField("correlation_id", getCorrelationID(resp, correlationID))
 
 	const baseLogText = "Resetting runner authentication token..."
 	switch result {
@@ -579,6 +614,7 @@ func (n *GitLabClient) RequestJob(
 
 	var response common.JobResponse
 
+	headers, correlationID := addCorrelationID(RunnerTokenHeader(config.Token))
 	//nolint:bodyclose
 	result, statusText, httpResponse := n.doMeasuredJSON(
 		ctx,
@@ -591,13 +627,13 @@ func (n *GitLabClient) RequestJob(
 			method:      http.MethodPost,
 			uri:         "jobs/request",
 			statusCode:  http.StatusCreated,
-			headers:     RunnerTokenHeader(config.Token),
+			headers:     headers,
 			request:     &request, response: &response,
 		},
 	)
-	defer func() { n.handleResponse(ctx, httpResponse, false) }()
+	defer closeResponseBody(httpResponse, false)
 
-	logger := config.Log().WithField("correlation_id", getCorrelationId(httpResponse))
+	logger := config.Log().WithField("correlation_id", getCorrelationID(httpResponse, correlationID))
 
 	switch result {
 	case http.StatusCreated:
@@ -613,7 +649,7 @@ func (n *GitLabClient) RequestJob(
 		}
 		addTLSData(&response, tlsData)
 
-		response.JobRequestCorrelationID = getCorrelationId(httpResponse)
+		response.JobRequestCorrelationID = getCorrelationID(httpResponse, correlationID)
 
 		return &response, true
 	case http.StatusForbidden:
@@ -622,6 +658,9 @@ func (n *GitLabClient) RequestJob(
 	case http.StatusNoContent:
 		logger.WithField("status", statusText).Debug("Checking for jobs...", "no content")
 		return nil, true
+	case http.StatusServiceUnavailable:
+		logger.WithField("status", statusText).Warningln("Checking for jobs...", "GitLab instance currently unavailable")
+		return nil, true
 	case clientError:
 		logger.WithField("status", statusText).Errorln("Checking for jobs...", "client error")
 		return nil, false
@@ -654,6 +693,7 @@ func (n *GitLabClient) UpdateJob(
 
 	log.Info("Updating job...")
 
+	headers, correlationID := addCorrelationID(JobTokenHeader(jobCredentials.Token))
 	//nolint:bodyclose
 	statusCode, statusText, response := n.doMeasuredJSON(
 		context.Background(),
@@ -666,13 +706,13 @@ func (n *GitLabClient) UpdateJob(
 			method:      http.MethodPut,
 			uri:         fmt.Sprintf("jobs/%d", jobInfo.ID),
 			statusCode:  http.StatusOK,
-			headers:     JobTokenHeader(jobCredentials.Token),
+			headers:     headers,
 			request:     &request,
 			response:    nil,
 		},
 	)
 
-	return n.createUpdateJobResult(log, statusCode, statusText, response)
+	return n.createUpdateJobResult(log, statusCode, statusText, response, correlationID)
 }
 
 func (n *GitLabClient) createUpdateJobResult(
@@ -680,8 +720,9 @@ func (n *GitLabClient) createUpdateJobResult(
 	statusCode int,
 	statusText string,
 	response *http.Response,
+	fallbackCorrelationID string,
 ) common.UpdateJobResult {
-	defer func() { n.handleResponse(context.TODO(), response, false) }()
+	defer closeResponseBody(response, false)
 
 	remoteJobStateResponse := NewRemoteJobStateResponse(response, log)
 
@@ -694,7 +735,7 @@ func (n *GitLabClient) createUpdateJobResult(
 		"code":            statusCode,
 		"job-status":      remoteJobStateResponse.RemoteState,
 		"update-interval": remoteJobStateResponse.RemoteUpdateInterval,
-		"correlation_id":  getCorrelationId(response),
+		"correlation_id":  getCorrelationID(response, fallbackCorrelationID),
 	})
 
 	switch {
@@ -747,6 +788,7 @@ func (n *GitLabClient) PatchTrace(
 
 	headers := JobTokenHeader(jobCredentials.Token)
 	headers.Set("Content-Range", contentRange)
+	headers, correlationID := addCorrelationID(headers)
 
 	bodyProvider := common.BytesProvider{Data: content}
 
@@ -770,7 +812,7 @@ func (n *GitLabClient) PatchTrace(
 		return common.NewPatchTraceResult(startOffset, common.PatchFailed, 0)
 	}
 
-	defer func() { n.handleResponse(context.TODO(), response, true) }()
+	defer closeResponseBody(response, true)
 
 	tracePatchResponse := NewTracePatchResponse(response, baseLog)
 	log := baseLog.WithFields(logrus.Fields{
@@ -780,7 +822,7 @@ func (n *GitLabClient) PatchTrace(
 		"code":            response.StatusCode,
 		"status":          response.Status,
 		"update-interval": tracePatchResponse.RemoteUpdateInterval,
-		"correlation_id":  getCorrelationId(response),
+		"correlation_id":  getCorrelationID(response, correlationID),
 	})
 
 	return n.createPatchTraceResult(startOffset, tracePatchResponse, response, endOffset, log)
@@ -931,6 +973,8 @@ func (n *GitLabClient) UploadRawArtifacts(
 
 	query := uploadRawArtifactsQuery(options)
 
+	headers, correlationID := addCorrelationID(JobTokenHeader(config.Token))
+
 	res, err := n.doRaw(
 		context.Background(),
 		&config,
@@ -938,14 +982,15 @@ func (n *GitLabClient) UploadRawArtifacts(
 		fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode()),
 		bodyProvider,
 		contentType,
-		JobTokenHeader(config.Token))
+		headers,
+	)
 
-	defer func() { n.handleResponse(context.TODO(), res, true) }()
+	defer closeResponseBody(res, true)
 
 	log := logrus.WithFields(logrus.Fields{
 		"id":             config.ID,
 		"token":          helpers.ShortenToken(config.Token),
-		"correlation_id": getCorrelationId(res),
+		"correlation_id": getCorrelationID(res, correlationID),
 	})
 
 	if options.LogResponseDetails {
@@ -1061,6 +1106,7 @@ func (n *GitLabClient) DownloadArtifacts(
 
 	uri := fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode())
 
+	headers, correlationID := addCorrelationID(JobTokenHeader(config.Token))
 	res, err := n.doRaw(
 		context.Background(),
 		&config,
@@ -1068,12 +1114,13 @@ func (n *GitLabClient) DownloadArtifacts(
 		uri,
 		nil,
 		"",
-		JobTokenHeader(config.Token))
+		headers,
+	)
 
 	log := logrus.WithFields(logrus.Fields{
 		"id":             config.ID,
 		"token":          helpers.ShortenToken(config.Token),
-		"correlation_id": getCorrelationId(res),
+		"correlation_id": getCorrelationID(res, correlationID),
 	})
 
 	if res != nil {
@@ -1088,7 +1135,7 @@ func (n *GitLabClient) DownloadArtifacts(
 		log.Errorln("Downloading artifacts from coordinator...", "error", err.Error())
 		return common.DownloadFailed
 	}
-	defer func() { n.handleResponse(context.TODO(), res, true) }()
+	defer closeResponseBody(res, true)
 
 	switch res.StatusCode {
 	case http.StatusOK:
@@ -1138,35 +1185,21 @@ func (n *GitLabClient) ProcessJob(
 ) (common.JobTrace, error) {
 	trace, err := newJobTrace(n, config, jobCredentials)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("create job trace: %w", err)
 	}
 
 	trace.start()
 	return trace, nil
 }
 
-func (n *GitLabClient) handleResponse(ctx context.Context, res *http.Response, discardBody bool) {
+func closeResponseBody(res *http.Response, discardBody bool) {
 	if res == nil {
 		return
 	}
-
-	defer func() {
-		if discardBody {
-			_, _ = io.Copy(io.Discard, io.LimitReader(res.Body, 1025*1025))
-		}
-		_ = res.Body.Close()
-	}()
-
-	if res.StatusCode != http.StatusTooManyRequests {
-		return
-	}
-
-	if retryAfter, err := strconv.Atoi(res.Header.Get(retryAfterHeader)); err == nil {
-		select {
-		case <-ctx.Done():
-		case <-time.After(time.Duration(retryAfter) * time.Second):
-		}
+	if discardBody {
+		_, _ = io.Copy(io.Discard, io.LimitReader(res.Body, 1025*1025))
 	}
+	_ = res.Body.Close()
 }
 
 func NewGitLabClientWithAPIRequestsCollector(c *APIRequestsCollector) *GitLabClient {
@@ -1179,10 +1212,9 @@ func NewGitLabClient() *GitLabClient {
 	return NewGitLabClientWithAPIRequestsCollector(NewAPIRequestsCollector())
 }
 
-func getCorrelationId(resp *http.Response) string {
-	if resp == nil {
-		return ""
+func getCorrelationID(resp *http.Response, fallbackValue string) string {
+	if resp == nil || resp.Header.Get(correlationIDHeader) == "" {
+		return fallbackValue
 	}
-
 	return resp.Header.Get(correlationIDHeader)
 }
